package org.moddingx.libx.impl.config.gui.screen;

import com.google.common.collect.ImmutableList;
import com.mojang.blaze3d.vertex.PoseStack;
import com.mojang.blaze3d.vertex.Tesselator;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.Font;
import net.minecraft.client.gui.components.AbstractWidget;
import net.minecraft.client.gui.components.Button;
import net.minecraft.client.gui.components.EditBox;
import net.minecraft.client.gui.components.events.GuiEventListener;
import net.minecraft.client.gui.narration.NarratableEntry;
import net.minecraft.client.gui.narration.NarrationElementOutput;
import net.minecraft.client.gui.screens.Screen;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.FormattedText;
import net.minecraft.util.FormattedCharSequence;
import net.minecraft.world.inventory.tooltip.TooltipComponent;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.client.gui.widget.ScrollPanel;
import org.apache.commons.lang3.tuple.Pair;
import org.lwjgl.glfw.GLFW;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;

import net.minecraft.client.gui.narration.NarratableEntry.NarrationPriority;

public abstract class ConfigBaseScreen extends Screen {

    protected final Minecraft mc;

    @Nullable
    private final ConfigScreenManager manager;
    private final boolean hasSearchBar;

    @Nullable
    private EditBox searchBar;
    
    @Nullable
    private BasePanel panel;
    
    // While rendering the scrollable view, tooltips must be delayed
    // Because scissors is enabled, and they need to be rendered with
    // absolute coordinates as they should not be cut by the screen border.
    private final List<Pair<PoseStack.Pose, Runnable>> capturedTooltips = new LinkedList<>();
    private boolean isCapturingTooltips = false;

    protected ConfigBaseScreen(Component title, @Nullable ConfigScreenManager manager, boolean hasSearchBar) {
        super(title);
        this.mc = Minecraft.m_91087_();
        this.manager = manager;
        this.hasSearchBar = hasSearchBar;
    }

    @Override
    protected void m_7856_() {
        if (this.manager != null) {
            Button back = new Button(5, 5, 42, 20, Component.m_237113_("\u2190 ").m_7220_(Component.m_237115_("libx.config.gui.back")), button -> this.manager.close());
            this.m_142416_(back);
        }

        if (this.hasSearchBar) {
            boolean shouldFocus = this.searchBar != null && this.searchBar.m_93696_();
            boolean isActive = this.searchBar != null && this.m_7222_() == this.searchBar;
            this.searchBar = new EditBox(this.mc.f_91062_, 20, 18 + this.mc.f_91062_.f_92710_, this.f_96543_ - 40, 20, this.searchBar, Component.m_237115_("libx.config.gui.search.title"));
            this.searchBar.m_94199_(32767);
            this.searchBar.m_94178_(shouldFocus);
            this.m_142416_(this.searchBar);
            if (isActive) {
                this.m_7522_(this.searchBar);
            }
            // Responder must be set last, or we'll trigger it while configuring the search bar
            this.searchBar.m_94151_(this::searchChange);
        } else {
            this.searchBar = null;
        }

        this.rebuild();
    }

    protected void rebuild() {
        if (this.panel != null) {
            this.m_169411_(this.panel);
        }
        
        ImmutableList.Builder<AbstractWidget> widgetBuilder = ImmutableList.builder();
        this.buildGui(widgetBuilder::add);
        List<AbstractWidget> widgets = widgetBuilder.build();

        int totalHeight = 10 + widgets.stream().map(w -> w.f_93621_ + w.m_93694_()).max(Comparator.naturalOrder()).orElse(0);
        int paddingTop = 18 + this.mc.f_91062_.f_92710_ + (this.hasSearchBar ? 26 : 0);

        this.panel = new BasePanel(this.mc, this.f_96543_ - 2, this.f_96544_ - paddingTop, paddingTop, 1) {

            @Override
            protected int getContentHeight() {
                return totalHeight;
            }

            @Override
            public void m_6305_(PoseStack poseStack, int mouseX, int mouseY, float partialTicks) {
                ConfigBaseScreen.this.isCapturingTooltips = true;
                super.m_6305_(poseStack, mouseX, mouseY, partialTicks);
                ConfigBaseScreen.this.isCapturingTooltips = false;
                ConfigBaseScreen.this.capturedTooltips.forEach(pair -> {
                    poseStack.m_85836_();
                    poseStack.m_166856_();
                    poseStack.m_166854_(pair.getLeft().m_85861_());
                    pair.getRight().run();
                    poseStack.m_85849_();
                });
                ConfigBaseScreen.this.capturedTooltips.clear();
            }

            @Override
            protected void drawPanel(PoseStack poseStack, int entryRight, int relativeY, Tesselator tess, int mouseX, int mouseY) {
                poseStack.m_85836_();
                poseStack.m_85837_(0, relativeY, 0);
                for (AbstractWidget widget : widgets) {
                    widget.m_6305_(poseStack, mouseX, mouseY - relativeY, ConfigBaseScreen.this.mc.m_91297_());
                }
                poseStack.m_85849_();
            }

            @Override
            public boolean m_6375_(double mouseX, double mouseY, int button) {
                // Without this, the panel would process clicks from areas currently not on the screen
                if (mouseX >= this.left && mouseX <= this.left + this.width && mouseY >= this.top && mouseY <= this.top + this.height) {
                    return super.m_6375_(mouseX, mouseY, button);
                } else {
                    return false;
                }
            }

            @Override
            protected boolean clickPanel(double mouseX, double mouseY, int button) {
                // Extra var required as we need to cal all listeners
                // so widgets can for example handle their loss of focus.
                boolean success = false;
                for (GuiEventListener widget : widgets) {
                    if (widget.m_6375_(mouseX, mouseY, button)) {
                        this.m_7522_(widget);
                        if (button == 0) {
                            this.m_7897_(true);
                        }
                        success = true;
                    }
                }
                return success;
            }

            @Override
            public boolean m_7979_(double mouseX, double mouseY, int button, double dragX, double dragY) {
                if (!super.m_7979_(mouseX, mouseY, button, dragX, dragY)) {
                    if (this.m_7222_() != null && this.m_7282_() && button == 0) {
                        return this.m_7222_().m_7979_(mouseX, mouseY - this.top + ((int) this.scrollDistance) - this.border, button, dragX, dragY);
                    } else {
                        return false;
                    }
                } else {
                    return true;
                }
            }

            @Override
            public boolean m_6348_(double mouseX, double mouseY, int button) {
                if (!super.m_6348_(mouseX, mouseY, button)) {
                    if (this.m_7222_() != null) {
                        return this.m_7222_().m_6348_(mouseX, mouseY - this.top + ((int) this.scrollDistance) - this.border, button);
                    } else {
                        return false;
                    }
                } else {
                    return true;
                }
            }
        };
        this.m_142416_(this.panel);
    }

    protected abstract void buildGui(Consumer<AbstractWidget> consumer);

    @Nullable
    public ConfigScreenManager getCurrentManager() {
        return this.manager;
    }

    @Override
    public void m_6305_(@Nonnull PoseStack poseStack, int mouseX, int mouseY, float partialTicks) {
        this.m_96626_(0);
        super.m_6305_(poseStack, mouseX, mouseY, partialTicks);
        //noinspection IntegerDivisionInFloatingPointContext
        this.mc.f_91062_.m_92763_(poseStack, this.m_96636_(), (this.f_96543_ - this.mc.f_91062_.m_92852_(this.m_96636_())) / 2, 12, 0xFFFFFF);
    }

    @Override
    public boolean m_7933_(int key, int i1, int i2) {
        if (key == GLFW.GLFW_KEY_ESCAPE && this.m_6913_() && this.manager != null) {
            this.manager.close();
            return true;
        } else {
            return super.m_7933_(key, i1, i2);
        }
    }

    public String searchTerm() {
        return this.searchBar == null ? "" : this.searchBar.m_94155_();
    }
    
    protected void searchChange(String term) {
        
    }

    private static abstract class BasePanel extends ScrollPanel implements NarratableEntry {

        public BasePanel(Minecraft mc, int width, int height, int top, int left) {
            super(mc, width, height, top, left);
        }

        @Nonnull
        @Override
        public NarrationPriority m_142684_() {
            return NarrationPriority.NONE;
        }

        @Override
        public void m_142291_(@Nonnull NarrationElementOutput output) {
            //
        }
    }

    private void captureTooltip(PoseStack.Pose pose, Runnable action) {
        this.capturedTooltips.add(Pair.of(pose, action));
    }

    @Override
    protected void m_6057_(@Nonnull PoseStack poseStack, @Nonnull ItemStack stack, int mouseX, int mouseY) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.m_6057_(poseStack, stack, mouseX, mouseY));
        } else {
            super.m_6057_(poseStack, stack, mouseX, mouseY);
        }
    }

    @Override
    public void renderTooltip(@Nonnull PoseStack poseStack, @Nonnull List<Component> components, @Nonnull Optional<TooltipComponent> special, int x, int y, @Nonnull ItemStack stack) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.renderTooltip(poseStack, components, special, x, y, stack));
        } else {
            super.renderTooltip(poseStack, components, special, x, y, stack);
        }
    }

    @Override
    public void renderTooltip(@Nonnull PoseStack poseStack, @Nonnull List<Component> components, @Nonnull Optional<TooltipComponent> special, int x, int y, @Nullable Font font) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.renderTooltip(poseStack, components, special, x, y, font));
        } else {
            super.renderTooltip(poseStack, components, special, x, y, font);
        }
    }

    @Override
    public void renderTooltip(@Nonnull PoseStack poseStack, @Nonnull List<Component> components, @Nonnull Optional<TooltipComponent> special, int x, int y, @Nullable Font font, @Nonnull ItemStack stack) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.renderTooltip(poseStack, components, special, x, y, font, stack));
        } else {
            super.renderTooltip(poseStack, components, special, x, y, font, stack);
        }
    }

    @Override
    public void m_169388_(@Nonnull PoseStack poseStack, @Nonnull List<Component> components, @Nonnull Optional<TooltipComponent> special, int mouseX, int mouseY) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.m_169388_(poseStack, components, special, mouseX, mouseY));
        } else {
            super.m_169388_(poseStack, components, special, mouseX, mouseY);
        }
    }

    @Override
    public void m_96602_(@Nonnull PoseStack poseStack, @Nonnull Component component, int mouseX, int mouseY) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.m_96602_(poseStack, component, mouseX, mouseY));
        } else {
            super.m_96602_(poseStack, component, mouseX, mouseY);
        }
    }

    @Override
    public void m_96597_(@Nonnull PoseStack poseStack, @Nonnull List<Component> components, int mouseX, int mouseY) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.m_96597_(poseStack, components, mouseX, mouseY));
        } else {
            super.m_96597_(poseStack, components, mouseX, mouseY);
        }
    }

    @Override
    public void renderComponentTooltip(@Nonnull PoseStack poseStack, @Nonnull List<? extends FormattedText> components, int mouseX, int mouseY, @Nonnull ItemStack stack) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.renderComponentTooltip(poseStack, components, mouseX, mouseY, stack));
        } else {
            super.renderComponentTooltip(poseStack, components, mouseX, mouseY, stack);
        }
    }

    @Override
    public void renderComponentTooltip(@Nonnull PoseStack poseStack, @Nonnull List<? extends FormattedText> components, int mouseX, int mouseY, @Nullable Font font) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.renderComponentTooltip(poseStack, components, mouseX, mouseY, font));
        } else {
            super.renderComponentTooltip(poseStack, components, mouseX, mouseY, font);
        }
    }

    @Override
    public void renderComponentTooltip(@Nonnull PoseStack poseStack, @Nonnull List<? extends FormattedText> components, int mouseX, int mouseY, @Nullable Font font, @Nonnull ItemStack stack) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.renderComponentTooltip(poseStack, components, mouseX, mouseY, font, stack));
        } else {
            super.renderComponentTooltip(poseStack, components, mouseX, mouseY, font, stack);
        }
    }

    @Override
    public void m_96617_(@Nonnull PoseStack poseStack, @Nonnull List<? extends FormattedCharSequence> components, int mouseX, int mouseY) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.m_96617_(poseStack, components, mouseX, mouseY));
        } else {
            super.m_96617_(poseStack, components, mouseX, mouseY);
        }
    }

    @Override
    public void renderTooltip(@Nonnull PoseStack poseStack, @Nonnull List<? extends FormattedCharSequence> components, int x, int y, @Nonnull Font font) {
        if (this.isCapturingTooltips) {
            // Not inside lambda as the value may change
            this.captureTooltip(poseStack.m_85850_(), () -> this.renderTooltip(poseStack, components, x, y, font));
        } else {
            super.renderTooltip(poseStack, components, x, y, font);
        }
    }
}
