package org.moddingx.libx.impl.datapack;

import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.packs.repository.Pack;
import net.minecraft.server.packs.repository.PackSource;
import net.minecraft.server.packs.repository.RepositorySource;
import net.minecraftforge.event.AddPackFindersEvent;
import net.minecraftforge.fml.ModList;
import net.minecraftforge.forgespi.language.IModFileInfo;
import org.moddingx.libx.LibX;

import javax.annotation.Nonnull;
import java.util.HashSet;
import java.util.Set;
import java.util.function.Consumer;

public class DynamicDatapackLocator implements RepositorySource {
    
    private static final DynamicDatapackLocator INSTANCE = new DynamicDatapackLocator();
    private static final Set<ResourceLocation> enabledPacks = new HashSet<>();
    
    private DynamicDatapackLocator() {
        
    }

    public static void locatePacks(AddPackFindersEvent event) {
        event.addRepositorySource(INSTANCE);
    }
    
    public static synchronized void enablePack(ResourceLocation id) {
        enabledPacks.add(id);
    }
    
    public static synchronized boolean isEnabled(ResourceLocation id) {
        return enabledPacks.contains(id);
    }
    
    @Override
    public void m_7686_(@Nonnull Consumer<Pack> packs, @Nonnull Pack.PackConstructor factory) {
        for (ResourceLocation id : enabledPacks) {
            String name = LibXDatapack.PREFIX + "/" + id.m_135827_() + ":" + id.m_135815_();
            IModFileInfo fileInfo = ModList.get().getModFileById(id.m_135827_());
            if (fileInfo == null || fileInfo.getFile() == null) {
                LibX.logger.warn("Can't create dynamic datapack " + id + ": Invalid mod file: " + fileInfo);
            } else {
                Pack pack = Pack.m_10430_(name, false,
                        () -> new LibXDatapack(fileInfo.getFile(), id.m_135815_()), factory,
                        Pack.Position.BOTTOM, PackSource.f_10527_
                );
                if (pack != null) {
                    packs.accept(pack);
                }
            }
        }
    }
}
