package org.moddingx.libx.render;

import com.mojang.blaze3d.vertex.DefaultVertexFormat;
import com.mojang.blaze3d.vertex.PoseStack;
import com.mojang.blaze3d.vertex.VertexConsumer;
import com.mojang.blaze3d.vertex.VertexFormat;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.MultiBufferSource;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.client.renderer.block.model.BakedQuad;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.resources.model.BakedModel;
import net.minecraft.client.resources.model.ModelBakery;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.RandomSource;
import net.minecraft.world.inventory.InventoryMenu;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.RenderShape;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.client.model.data.ModelData;
import org.moddingx.libx.impl.render.BlockOverlayQuadCache;

import java.util.List;
import java.util.function.Predicate;

/**
 * Helper to deal with {@link Block} rendering
 */
public class RenderHelperBlock {

    private static final RenderType RENDER_TYPE_BREAK = RenderType.m_110494_(InventoryMenu.f_39692_);
    private static final RandomSource random = RandomSource.m_216327_();

    /**
     * Renders the break effect for a {@link BlockState}.
     *
     * @param breakProgress How much the block already broke. 0 means no break. This should not be lower than 0 and not be greater than 10.
     */
    public static void renderBlockBreak(BlockState state, PoseStack poseStack, int light, int overlay, int breakProgress) {
        renderBlockBreak(state, poseStack, light, overlay, breakProgress, state.m_60726_(BlockPos.f_121853_));
    }

    /**
     * Renders the break effect for a {@link BlockState}.
     *
     * @param breakProgress  How much the block already broke. 0 means no break. This should not be lower than 0 and not be greater than 10.
     * @param positionRandom The long value to randomize the position. This can be obtained via {@code BlockState#getPositionRandom}.
     */
    public static void renderBlockBreak(BlockState state, PoseStack poseStack, int light, int overlay, int breakProgress, long positionRandom) {
        if (breakProgress > 0) {
            ResourceLocation tex = ModelBakery.f_119227_.get((breakProgress - 1) % ModelBakery.f_119227_.size());
            TextureAtlasSprite sprite = Minecraft.m_91087_().m_91258_(InventoryMenu.f_39692_).apply(tex);
            renderBlockOverlaySprite(state, poseStack, light, overlay, sprite, positionRandom);
        }
    }

    /**
     * Renders a block overlay on top of a {@link BlockState} with the same method as the crumbling is rendered. However you cen specify
     * your own TextureAtlasSprite here to be used. (It must be from {@link InventoryMenu#BLOCK_ATLAS})
     */
    public static void renderBlockOverlaySprite(BlockState state, PoseStack poseStack, int light, int overlay, TextureAtlasSprite sprite) {
        renderBlockOverlaySprite(state, poseStack, light, overlay, sprite, state.m_60726_(BlockPos.f_121853_));
    }

    /**
     * Renders a block overlay on top of a {@link BlockState} with the same method as the crumbling is rendered. However you cen specify
     * your own TextureAtlasSprite here to be used. (It must be from {@link InventoryMenu#BLOCK_ATLAS})
     */
    public static void renderBlockOverlaySprite(BlockState state, PoseStack poseStack, int light, int overlay, TextureAtlasSprite sprite, long positionRandom) {
        renderBlockOverlaySprite(state, poseStack, light, overlay, sprite, positionRandom, dir -> true);
    }

    /**
     * Renders a block overlay on top of a {@link BlockState} with the same method as the crumbling is rendered. However you cen specify
     * your own TextureAtlasSprite here to be used. (It must be from {@link InventoryMenu#BLOCK_ATLAS})
     */
    public static void renderBlockOverlaySprite(BlockState state, PoseStack poseStack, int light, int overlay, TextureAtlasSprite sprite, Predicate<Direction> dirs) {
        renderBlockOverlaySprite(state, poseStack, light, overlay, sprite, state.m_60726_(BlockPos.f_121853_), dirs);
    }

    /**
     * Renders a block overlay on top of a {@link BlockState} with the same method as the crumbling is rendered. However you cen specify
     * your own TextureAtlasSprite here to be used. (It must be from {@link InventoryMenu#BLOCK_ATLAS})
     */
    public static void renderBlockOverlaySprite(BlockState state, PoseStack poseStack, int light, int overlay, TextureAtlasSprite sprite, long positionRandom, Predicate<Direction> dirs) {
        if (state.m_60799_() == RenderShape.MODEL) {
            BakedModel model = Minecraft.m_91087_().m_91289_().m_110907_().m_110893_(state);
            VertexConsumer vertex = Minecraft.m_91087_().m_91269_().m_110108_().m_6299_(RENDER_TYPE_BREAK);
            for (Direction direction : Direction.values()) {
                random.m_188584_(positionRandom);
                List<BakedQuad> list = model.getQuads(state, direction, random, ModelData.EMPTY, RENDER_TYPE_BREAK);
                if (!list.isEmpty()) {
                    renderBlockOverlayQuad(DefaultVertexFormat.f_85811_, poseStack.m_85850_(), vertex, list, light, overlay, sprite, dirs);
                }
            }

            random.m_188584_(positionRandom);
            List<BakedQuad> list = model.getQuads(state, null, random, ModelData.EMPTY, RENDER_TYPE_BREAK);
            if (!list.isEmpty()) {
                renderBlockOverlayQuad(DefaultVertexFormat.f_85811_, poseStack.m_85850_(), vertex, list, light, overlay, sprite, dirs);
            }
        }
    }

    /**
     * Invokes {@link MultiBufferSource.BufferSource#endBatch(RenderType)} on the crumbling buffer with the {@link RenderType}
     * used for custom overlays and crumbling.
     */
    public static void endOverlayBatch() {
        Minecraft.m_91087_().m_91269_().m_110108_().m_109912_(RENDER_TYPE_BREAK);
    }

    private static void renderBlockOverlayQuad(@SuppressWarnings("SameParameterValue") VertexFormat format, PoseStack.Pose pose, VertexConsumer vertex, List<BakedQuad> list, int light, int overlay, TextureAtlasSprite sprite, Predicate<Direction> dirs) {
        for (BakedQuad quad : list) {
            if (dirs.test(quad.m_111306_())) {
                vertex.m_85987_(pose, modifyBlockQuad(format, quad, sprite), 1, 1, 1, light, overlay);
            }
        }
    }

    private static BakedQuad modifyBlockQuad(VertexFormat format, BakedQuad quad, TextureAtlasSprite newSprite) {
        BakedQuad result = BlockOverlayQuadCache.get(quad, newSprite);
        if (result == null) {
            int[] data = quad.m_111303_();
            int[] newData = new int[data.length];
            System.arraycopy(data, 0, newData, 0, data.length);
            int uvIdx = format.m_86023_().indexOf(DefaultVertexFormat.f_166849_);
            if (uvIdx != -1) {
                // Byte offset / 4 = integer offset
                // Will break if UV is not on full ints but that is not the case in regular minecraft
                TextureAtlasSprite oldSprite = quad.m_173410_();
                int uvOff = format.getOffset(uvIdx) / 4;
                int intSize = format.m_86017_();
                for (int off = 0; off + uvOff + 1 < newData.length; off += intSize) {
                    newData[off + uvOff] = Float.floatToRawIntBits(((Float.intBitsToFloat(data[off + uvOff]) - oldSprite.m_118409_()) * newSprite.m_118405_() / oldSprite.m_118405_()) + newSprite.m_118409_());
                    newData[off + uvOff + 1] = Float.floatToRawIntBits(((Float.intBitsToFloat(data[off + uvOff + 1]) - oldSprite.m_118411_()) * newSprite.m_118408_() / oldSprite.m_118408_()) + newSprite.m_118411_());
                }
            }
            result = new BakedQuad(newData, quad.m_111305_(), quad.m_111306_(), newSprite, quad.m_111307_());
            BlockOverlayQuadCache.put(quad, result);
        }
        return result;
    }
}
