package org.moddingx.libx.screen;

import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.components.AbstractWidget;
import net.minecraft.client.gui.components.Widget;
import net.minecraft.client.gui.components.events.GuiEventListener;
import net.minecraft.client.gui.narration.NarrationElementOutput;
import net.minecraft.client.gui.screens.Screen;
import net.minecraft.network.chat.Component;
import org.moddingx.libx.config.gui.EditorOps;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

/**
 * An {@link AbstractWidget} that is composed of multiple other widgets. These
 * widgets are positioned relative to this widget. You can add these widgets in
 * the constructor.
 */
public abstract class Panel extends AbstractWidget implements EditorOps {

    protected final Screen screen;
    private final List<GuiEventListener> children = new ArrayList<>();
    private final List<Widget> renderables = new ArrayList<>();

    @Nullable
    protected GuiEventListener focused = null;

    public Panel(Screen screen, int x, int y, int width, int height) {
        super(x, y, width, height, Component.m_237119_());
        this.screen = screen;
    }

    /**
     * Adds a widget that can be rendered.
     */
    protected <T extends GuiEventListener & Widget> T addRenderableWidget(T widget) {
        this.renderables.add(widget);
        this.children.add(widget);
        return widget;
    }
    
    /**
     * Adds a component that can be rendered.
     */
    protected <T extends Widget> T addRenderableOnly(T widget) {
        this.renderables.add(widget);
        return widget;
    }
    
    /**
     * Adds a widget to listen to events.
     */
    protected <T extends GuiEventListener> T addWidget(T widget) {
        this.children.add(widget);
        return widget;
    }

    @Override
    public void m_6305_(@Nonnull PoseStack poseStack, int mouseX, int mouseY, float partialTicks) {
        poseStack.m_85836_();
        poseStack.m_85837_(this.f_93620_, this.f_93621_, 0);
        for (Widget widget : this.renderables) {
            widget.m_6305_(poseStack, mouseX - this.f_93620_, mouseY - this.f_93621_, partialTicks);
        }
        poseStack.m_85849_();
    }

    @Override
    public boolean m_6375_(double mouseX, double mouseY, int button) {
        boolean success = false;
        for (GuiEventListener child : this.children) {
            if (child.m_6375_(mouseX - this.f_93620_, mouseY - this.f_93621_, button)) {
                this.screen.m_7522_(this);
                this.focused = child;
                if (button == 0) {
                    this.screen.m_7897_(true);
                }
                success = true;
            }
        }
        return success;
    }

    @Override
    public boolean m_6348_(double mouseX, double mouseY, int button) {
        this.screen.m_7897_(false);
        for (GuiEventListener child : this.children) {
            if (child.m_5953_(mouseX - this.f_93620_, mouseY - this.f_93621_)) {
                if (child.m_6348_(mouseX - this.f_93620_, mouseY - this.f_93621_, button)) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public boolean m_7979_(double mouseX, double mouseY, int button, double dragX, double dragY) {
        return this.focused != null && this.screen.m_7282_() && this.focused.m_7979_(mouseX - this.f_93620_, mouseY - this.f_93621_, button, dragX - this.f_93620_, dragY - this.f_93621_);
    }

    @Override
    public boolean m_7933_(int keyCode, int scanCode, int modifiers) {
        return this.focused != null && this.focused.m_7933_(keyCode, scanCode, modifiers);
    }

    @Override
    public boolean m_7920_(int keyCode, int scanCode, int modifiers) {
        return this.focused != null && this.focused.m_7920_(keyCode, scanCode, modifiers);
    }

    @Override
    public boolean m_5534_(char value, int modifiers) {
        return this.focused != null && this.focused.m_5534_(value, modifiers);
    }

    @Override
    public void m_142291_(@Nonnull NarrationElementOutput output) {
        //
    }

    @Override
    public void enabled(boolean enabled) {
        for (Widget child : this.renderables) {
            EditorOps.wrap(child).enabled(enabled);
        }
    }
}
